<?php
/*
Plugin Name: Wordpress MU Demo Data Creator
Plugin URI: http://www.stillbreathing.co.uk/projects/demodata/
Description: A plugin for Wordpress MU and BuddyPress which will allow a site administrator to create multiple fake users and blogs to test their installation. This is very useful when developing plugins and themes that you need to work with many users and blogs.
Version: 0.2
Author: Chris Taylor
Author URI: http://www.stillbreathing.co.uk
*/

// if the file is being called outside of Wordpress
if (!function_exists('add_action'))
{
	// include Wordpress
	require($_SERVER['DOCUMENT_ROOT'] . '/wp-blog-header.php');

	// listen for form submission
	demodata_watch_form();
}

// when the admin menu is built
add_action('admin_menu', 'demodata_add_menu_items');

// ======================================================
// Admin functions
// ======================================================

// create the data
function demodata_create()
{
	// listen for a form submission
	if (count($_POST) > 0 && isset($_POST["create"]))
	{
	
		switch ($_POST["create"]) {
		
			// creating users
			case "users":
				demodata_create_users();
				break;
		
			// creating blogs
			case "blogs":
				demodata_create_blogs();
				break;
		
			// creating categories
			case "categories":
				demodata_create_categories();
				break;
		
			// creating posts
			case "posts":
				demodata_create_posts();
				break;
				
			// creating pages
			case "pages":
				demodata_create_pages();
				break;
				
			// creating comments
			case "comments":
				demodata_create_comments();
				break;
				
			// creating links
			case "links":
				demodata_create_links();
				break;
				
			// creating groups
			case "groups":
				demodata_create_groups();
				break;
				
			// creating wire messages
			case "wire":
				// creating demo wire messages disabled as I can't get it to work
				//demodata_create_wire();
				break;
				
			// creating friends
			case "friends":
				demodata_create_friends();
				break;
		
		}
		
	}
}

// add the menu items to the Site Admin list
function demodata_add_menu_items()
{
	if (is_site_admin())
	{
		add_action("admin_head", "demodata_css");
		add_submenu_page('wpmu-admin.php', 'Demo Data Creator', 'Demo Data Creator', 10, 'demodata_form', 'demodata_form');
	}
}

// add the CSS to the admin page head
function demodata_css()
{
	if (isset($_GET["page"]) && $_GET["page"] == "demodata_form")
	{
		echo '
		<style type="text/css">
		html body form.demodata fieldset p {
		border-width: 0 0 1px 0;
		border-color: #AAA;
		}
		html body form.demodata fieldset label {
		float: left;
		width: 32em;
		}
		html body form.demodata fieldset input {
		width: 6em;
		}
		html body form.demodata fieldset input.text {
		width: 18em;
		}
		html body div.demodatasuccess {
		padding: 0 1em 1em 1em;
		margin-top: 1em;
		background: #D2FFCF;
		border: 1px solid #188F11;
		}
		html body div.demodataerror {
		padding: 0 1em 1em 1em;
		margin-top: 1em;
		background: #FFCFCF;
		border: 1px solid #BF0B0B;
		}
		</style>
		';
	}
}

// create demo users
function demodata_create_users()
{
	global $wpdb;
	global $current_site;

	// get the users settings
	$users = @$_POST["users"] == "" ? 100 : (int)$_POST["users"];
	$users = $users > 1000 ? $users = 1000 : $users = $users;
	$useremailtemplate = @$_POST["useremailtemplate"] == "" ? "demouser[x]@" . $current_site->domain : $_POST["useremailtemplate"];		
	
	// check all the settings
	if (
		$users != "" &&
		$useremailtemplate != ""
	)
	{
		$go = true;
	} else {
		$go = false;
	}
	
	// if the settings are OK
	if ($go)
	{
	
		// turn off new registration notifications
		$registrationnotification = get_site_option("registrationnotification");
		update_site_option("registrationnotification", "no");
	
		$success = true;
		$userx = 0;
		
		// loop the number of required users
		for($u = 0; $u < $users; $u++)
		{
			$userx++;
			
			// generate the details for this user
			// get a random name
			$username = demodata_firstname() . " " . demodata_lastname();
			$email = str_replace("[x]", $userx, $useremailtemplate);
			$random_password = substr(md5(uniqid(microtime())), 0, 6);
			
			// check the user can be created
			if (!wp_create_user($username, $random_password, $email) > 0)
			{
				$success = false;
				$error = "Error creating user " . $userx;
				$userx--;
				// break out of this loop
				break;
			}
		}
		
		// turn registration notification back on
		update_site_option("registrationnotification", $registrationnotification);
	
		if ($success)
		{
		
			echo '
			<div class="demodatasuccess">
			<p>' . $userx . " " . __("demo users created") . '</p>
			<p><a href="http://' . $current_site->domain . '">' . __("Your system is now ready for testing here.") . '</a></p>
			</div>
			';
			
		} else {
		
			echo '
			<div class="demodataerror">
			<p>' . $userx . ' ' . __("demo users created") . '</p>
			<p>' . __("Errors encountered:") . ' ' . $error . '</p>
			<p><a href="http://' . $current_site->domain . '">' . __("You can test this data in your system here.") . '</a></p>
			</div>
			';
			
		}
	} else {
	
		echo '
		<div class="demodataerror">
		<p>' . __("Some of your settings were not valid. Please check all the settings below.") . '</p>
		</div>
		';
	
	}
}

// create demo blogs
function demodata_create_blogs()
{
	global $wpdb;
	global $current_site;
	
	// get the blog settings
	$membershiptype = @$_POST["membershiptype"] == "" ? 1 : (int)$_POST["membershiptype"];
	$maxblogsperuser = @$_POST["maxblogsperuser"] == "" ? 1 : (int)$_POST["maxblogsperuser"];
	$maxblogsperuser = $maxblogsperuser > 5 ? $maxblogsperuser = 5 : $maxblogsperuser = $maxblogsperuser;
	
	// check all the settings
	if (
		$membershiptype != "" && 
		$maxblogsperuser != ""
	)
	{
		$go = true;
	} else {
		$go = false;
	}
	
	// if the settings are OK
	if ($go)
	{
		
		// turn off new registration notifications
		$registrationnotification = get_site_option("registrationnotification");
		update_site_option("registrationnotification", "no");
	
		$success = true;
		$blogx = 0;
				
		// set the minimum number of blogs
		if ($membershiptype == 1)
		{
			$minblogs = 1;
		} else {
			$minblogs = 0;
		}
		
		// get users
		$sql = "select id from " . $wpdb->users . ";";
		$users = $wpdb->get_results($sql);

		// loop users
		foreach($users as $user)
		{
		
			// get a random number of blogs
			$blogs = rand($minblogs, $maxblogsperuser);
			
			// loop the number of required blogs
			for($b = 0; $b < $blogs; $b++)
			{
			
				$blogx++;
				
				// get a random blogname
				$blogname = demodata_blogname();
				$blogdomain = "demoblog" . $blogx;

				// check the blog can be created
				if (!demodata_create_blog($blogx, $blogdomain, $blogname, $user->id))
				{
					$success = false;
					$error = "Error creating blog " . $blogx;
					$blogx--;
					// break out of this loop
					break;
				}
				
			}
			
		}
		
		// turn registration notification back on
		update_site_option("registrationnotification", $registrationnotification);
	
		if ($success)
		{
		
			echo '
			<div class="demodatasuccess">
			<p>' . $blogx . " " . __("demo blogs created") . '</p>
			<p><a href="http://' . $current_site->domain . '">' . __("Your system is now ready for testing here.") . '</a></p>
			</div>
			';
			
		} else {
		
			echo '
			<div class="demodataerror">
			<p>' . $blogx . ' ' . __("demo blogs created") . '</p>
			<p>' . __("Errors encountered:") . ' ' . $error . '</p>
			<p><a href="http://' . $current_site->domain . '">' . __("You can test this data in your system here.") . '</a></p>
			</div>
			';
			
		}
	} else {
	
		echo '
		<div class="demodataerror">
		<p>' . __("Some of your settings were not valid. Please check all the settings below.") . '</p>
		</div>
		';
	
	}
}

// create demo categories
function demodata_create_categories()
{
	global $wpdb;
	global $current_site;
	
	// get the categories and tags settings
	$maxblogcategories = @$_POST["maxblogcategories"] == "" ? 10 : (int)$_POST["maxblogcategories"];
	$maxblogcategories = $maxblogcategories > 25 ? $maxblogcategories = 25 : $maxblogcategories = $maxblogcategories;
	$maxblogtags = $maxblogtags > 100 ? $maxblogtags = 100 : $maxblogtags = $maxblogtags;
	
	// check all the settings
	if (
		$maxblogcategories != ""
	)
	{
		$go = true;
	} else {
		$go = false;
	}
	
	// if the settings are OK
	if ($go)
	{
	
		$categoryx = 0;
		$success = true;
		
		// get blogs
		$sql = "select blog_id, domain from " . $wpdb->blogs . ";";
		$blogs = $wpdb->get_results($sql);
		
		// loop blogs
		foreach($blogs as $blog)
		{
		
			// switch to this blog
			switch_to_blog($blog->blog_id);
		
			// get a random number of blog categories
			$categories = rand(0, $maxblogcategories);
			
			// loop the number of required categories
			for($c = 0; $c < $categories; $c++)
			{
			
				$categoryx++;

				// see if the category can be inserted
				if (!demodata_create_category($blog->domain, $categoryx))
				{
					$categoryx--;
				}			
			}
			
			// switch back to the main blog
			restore_current_blog();
		}
	
		if ($success)
		{
		
			echo '
			<div class="demodatasuccess">
			<p>' . $categoryx . " " . __("demo categories created") . '</p>
			<p><a href="http://' . $current_site->domain . '">' . __("Your system is now ready for testing here.") . '</a></p>
			</div>
			';
			
		} else {
		
			echo '
			<div class="demodataerror">
			<p>' . $categoryx . ' ' . __("demo categories created") . '</p>
			<p>' . __("Errors encountered:") . ' ' . $error . '</p>
			<p><a href="http://' . $current_site->domain . '">' . __("You can test this data in your system here.") . '</a></p>
			</div>
			';
			
		}
	} else {
	
		echo '
		<div class="demodataerror">
		<p>' . __("Some of your settings were not valid. Please check all the settings below.") . '</p>
		</div>
		';
	
	}
}

// create demo posts
function demodata_create_posts()
{
	global $wpdb;
	global $current_site;
	
	// get the post settings
	$maxblogposts = @$_POST["maxblogposts"] == "" ? 50 : (int)$_POST["maxblogposts"];
	$maxblogposts = $maxblogposts > 100 ? $maxblogposts = 100 : $maxblogposts = $maxblogposts;
	$maxpostlength = @$_POST["maxpostlength"] == "" ? 10 : (int)$_POST["maxpostlength"];
	$maxpostlength = $maxpostlength > 50 ? $maxpostlength = 50 : $maxpostlength = $maxpostlength;
	$maxpostlength = $maxpostlength < 1 ? $maxpostlength = 1 : $maxpostlength = $maxpostlength;
	
	// check all the settings
	if (
		$maxblogposts != "" &&
		$maxpostlength != ""
	)
	{
		$go = true;
	} else {
		$go = false;
	}
	
	// if the settings are OK
	if ($go)
	{
	
		$postx = 0;
		$success = true;
		
		// get blogs
		$sql = "select blog_id, domain from " . $wpdb->blogs . ";";
		$blogs = $wpdb->get_results($sql);
		
		// loop blogs
		foreach($blogs as $blog)
		{
		
			// switch to this blog
			switch_to_blog($blog->blog_id);
		
			// get a random number of blog posts
			$posts = rand(0, $maxblogposts);
			
			// loop the number of required posts
			for($p = 0; $p < $posts; $p++)
			{
			
				$postx++;

				// see if the post can be inserted
				if (!demodata_create_post($blog->domain, $maxpostlength, $postx))
				{
					$postx--;
				}			
			}
			
			// switch back to the main blog
			restore_current_blog();
		}
	
		if ($success)
		{
		
			echo '
			<div class="demodatasuccess">
			<p>' . $postx . " " . __("demo posts created") . '</p>
			<p><a href="http://' . $current_site->domain . '">' . __("Your system is now ready for testing here.") . '</a></p>
			</div>
			';
			
		} else {
		
			echo '
			<div class="demodataerror">
			<p>' . $postx . ' ' . __("demo posts created") . '</p>
			<p>' . __("Errors encountered:") . ' ' . $error . '</p>
			<p><a href="http://' . $current_site->domain . '">' . __("You can test this data in your system here.") . '</a></p>
			</div>
			';
			
		}
	} else {
	
		echo '
		<div class="demodataerror">
		<p>' . __("Some of your settings were not valid. Please check all the settings below.") . '</p>
		</div>
		';
	
	}
}

// create demo pages
function demodata_create_pages()
{
	global $wpdb;
	global $current_site;

	// get the pages settings
	$maxpages = @$_POST["maxpages"] == "" ? 25 : (int)$_POST["maxpages"];
	$maxpages = $maxpages > 25 ? $maxpages = 25 : $maxpages = $maxpages;
	$maxtoppages = @$_POST["maxtoppages"] == "" ? 5 : (int)$_POST["maxtoppages"];
	$maxtoppages = $maxtoppages > 5 ? $maxtoppages = 5 : $maxtoppages = $maxtoppages;
	$maxpageslevels = @$_POST["maxpageslevels"] == "" ? 3 : (int)$_POST["maxpageslevels"];
	$maxpageslevels = $maxpageslevels > 3 ? $maxpageslevels = 3 : $maxpageslevels = $maxpageslevels;
	$maxpagelength = @$_POST["maxpagelength"] == "" ? 10 : (int)$_POST["maxpagelength"];
	$maxpagelength = $maxpagelength > 10 ? $maxpagelength = 10 : $maxpagelength = $maxpagelength;
	
	// check all the settings
	if (
		$maxpages != ""
		&& $maxtoppages != ""
		&& $maxpageslevels != ""
		&& $maxpagelength != ""
	)
	{
		$go = true;
	} else {
		$go = false;
	}
	
	// if the settings are OK
	if ($go)
	{
	
		$pagex = 0;
		$success = true;
		
		// get blogs
		$sql = "select blog_id, domain from " . $wpdb->blogs . ";";
		$blogs = $wpdb->get_results($sql);
		
		// loop blogs
		foreach($blogs as $blog)
		{
		
			// switch to this blog
			switch_to_blog($blog->blog_id);
			
			// get a random number of top pages
			$toppages = rand(1, $maxtoppages);
			
			// loop the number of top pages
			for($p = 0; $p < $toppages; $p++)
			{
				$pagex++;
				
				$id = demodata_create_page($blog->domain, 0, $maxpagelength, $pagex);
				
				if (!$id)
				{
					$pagex--;
				} else {
					// add the page id to the array
					$pageids[0][] = $id;
				}
			}
			
			// get random number of sublevels
			$levels = rand(1, $maxpageslevels);
			
			// if the levels is greater than 1
			if ($levels > 1)
			{
				// loop the top level pages
				foreach($pageids[0] as $pageid)
				{
					$pagex++;
				
					$id = demodata_create_page($blog->domain, $pageid, $maxpagelength, $pagex);
					
					if (!$id)
					{
						$pagex--;
					} else {
						// add the page id to the array
						$pageids[1][] = $id;
					}
				}
			}
			
			// if the levels is greater than 2
			if ($levels > 2)
			{
				// loop the level 1 pages
				foreach($pageids[1] as $pageid)
				{
					$pagex++;
				
					$id = demodata_create_page($blog->domain, $pageid, $maxpagelength, $pagex);
					
					if (!$id)
					{
						$pagex--;
					}
				}
			}
			
			// switch back to the main blog
			restore_current_blog();
		}
	
		if ($success)
		{
		
			echo '
			<div class="demodatasuccess">
			<p>' . $pagex . " " . __("demo pages created") . '</p>
			<p><a href="http://' . $current_site->domain . '">' . __("Your system is now ready for testing here.") . '</a></p>
			</div>
			';
			
		} else {
		
			echo '
			<div class="demodataerror">
			<p>' . $pagex . ' ' . __("demo pages created") . '</p>
			<p>' . __("Errors encountered:") . ' ' . $error . '</p>
			<p><a href="http://' . $current_site->domain . '">' . __("You can test this data in your system here.") . '</a></p>
			</div>
			';
			
		}
	
	} else {
	
		echo '
		<div class="demodataerror">
		<p>' . __("Some of your settings were not valid. Please check all the settings below.") . '</p>
		</div>
		';
	
	}
}

// create demo comments
function demodata_create_comments()
{
	global $wpdb;
	global $current_site;
	
	// get the comments settings
	$maxcomments = @$_POST["maxcomments"] == "" ? 50 : (int)$_POST["maxcomments"];
	$maxcomments = $maxcomments > 50 ? $maxcomments = 50 : $maxcomments = $maxcomments;
	
	// check all the settings
	if (
		$maxcomments != ""
	)
	{
		$go = true;
	} else {
		$go = false;
	}
	
	// if the settings are OK
	if ($go)
	{
	
		$commentx = 0;
		$success = true;
		
		// get blogs
		$sql = "select blog_id, domain from " . $wpdb->blogs . ";";
		$blogs = $wpdb->get_results($sql);
		
		// loop blogs
		foreach($blogs as $blog)
		{
		
			// switch to this blog
			switch_to_blog($blog->blog_id);
		
			// get posts
			$sql = "select id from " . $wpdb->posts . ";";
			$posts = $wpdb->get_results($sql);
			
			// loop posts
			foreach($posts as $post)
			{
			
				// get a random number of comments
				$comments = rand(0, $maxcomments);
				
				// loop the number of required comments
				for($c = 0; $c < $comments; $c++)
				{
					$commentx++;
					
					// see if the comment can be inserted
					if (!demodata_create_comment($blog->domain, $post->id, $commentx))
					{
						// continue
						$commentx--;
					}
				}
				
			}
			
			// switch back to the main blog
			restore_current_blog();
		}
	
		if ($success)
		{
		
			echo '
			<div class="demodatasuccess">
			<p>' . $commentx . ' ' . __("demo comments created") . '</p>
			<p><a href="http://' . $current_site->domain . '">' . __("Your system is now ready for testing here.") . '</a></p>
			</div>
			';
			
		} else {
		
			echo '
			<div class="demodataerror">
			<p>' . $commentx . ' ' . __("demo comments created") . '</p>
			<p>' . __("Errors encountered:") . ' ' . $error . '</p>
			<p><a href="http://' . $current_site->domain . '">' . __("You can test this data in your system here.") . '</a></p>
			</div>
			';
			
		}
	} else {
	
		echo '
		<div class="demodataerror">
		<p>' . __("Some of your settings were not valid. Please check all the settings below.") . '</p>
		</div>
		';
	
	}
}

// create demo links
function demodata_create_links()
{
	global $wpdb;
	global $current_site;
	
	// get the links settings
	$maxbloglinks = @$_POST["maxbloglinks"] == "" ? 25 : (int)$_POST["maxbloglinks"];
	$maxbloglinks = $maxbloglinks > 100 ? $maxbloglinks = 100 : $maxbloglinks = $maxbloglinks;
	
	// check all the settings
	if (
		$maxbloglinks != ""
	)
	{
		$go = true;
	} else {
		$go = false;
	}
	
	// if the settings are OK
	if ($go)
	{
	
		$linkx = 0;
		$success = true;
		
		// get blogs
		$sql = "select blog_id, domain from " . $wpdb->blogs . ";";
		$blogs = $wpdb->get_results($sql);
		
		// loop blogs
		foreach($blogs as $blog)
		{
		
			// switch to this blog
			switch_to_blog($blog->blog_id);
		
			// get a random number of bookmarks
			$links = rand(0, $maxbloglinks);
			
			// loop the number of required bookmarks
			for($l = 0; $l < $links; $l++)
			{
				$linkx++;
				if (!demodata_create_link($blog->domain, $linkx))
				{
					// continue
					$linkx--;
				}
			}
			
			// switch back to the main blog
			restore_current_blog();
		}
	
		if ($success)
		{
		
			echo '
			<div class="demodatasuccess">
			<p>' . $linkx . " " . __("demo links created") . '</p>
			<p><a href="http://' . $current_site->domain . '">' . __("Your system is now ready for testing here.") . '</a></p>
			</div>
			';
			
		} else {
		
			echo '
			<div class="demodataerror">
			<p>' . $linkx . ' ' . __("demo links created") . '</p>
			<p>' . __("Errors encountered:") . ' ' . $error . '</p>
			<p><a href="http://' . $current_site->domain . '">' . __("You can test this data in your system here.") . '</a></p>
			</div>
			';
			
		}
	} else {
	
		echo '
		<div class="demodataerror">
		<p>' . __("Some of your settings were not valid. Please check all the settings below.") . '</p>
		</div>
		';
	
	}
}

// create demo groups
function demodata_create_groups()
{
	// detect BuddyPress
	$buddypress = defined( 'BP_ROOT_BLOG' );
	
	if ($buddypress)
	{

		global $wpdb;
		global $current_site;
		
		// get groups settings
		$maxgroups = @$_POST["maxgroups"] == "" ? 250 : (int)$_POST["maxgroups"];
		$maxgroups = $maxgroups > 500 ? $maxgroups = 500 : $maxgroups = $maxgroups;
		$maxgroupmembership = @$_POST["maxgroupmembership"] == "" ? 25 : (int)$_POST["maxgroupmembership"];
		$maxgroupmembership = $maxgroupmembership > 50 ? $maxgroupmembership = 50 : $maxgroupmembership = $maxgroupmembership;
		
		// check all the settings
		if (
			$maxgroups != "" &&
			$maxgroupmembership != ""
		)
		{
			$go = true;
		} else {
			$go = false;
		}
		
		// if the settings are OK
		if ($go)
		{
		
			$groupx = 0;
			$memberx = 0;
			$success = true;
			
			// get a random number of groups
			$groups = rand(1, $maxgroups);
		
			// loop the number of required groups
			for($g = 0; $g < $groups; $g++)
			{
				$groupx++;
				
				// get a random group name
				$groupname = demodata_groupname();
				
				$newgroupid = demodata_create_group( $groupname, $groupsx );
				
				// save the group
				if ( !$newgroupid )
				{
				
					$success = false;
					$error = "Error creating group " . $groupx;
					$groupx--;
					// break out of this loop
					break;
					
				} else {
				
					$memberx++;

					// get a random number of group members
					$members = rand(1, $maxgroupmembership);
					
					$admin = demodata_create_group_member( $newgroupid, 1, $memberx );
					
					// if the admin member could not be saved
					if ( !$admin )
					{
					
						$success = false;
						$error = "Error creating group member " . $memberx;
						$memberx--;
						// break out of this loop
						break;
						
					} else {
					
						// loop the number of required members of this group
						for($m = 0; $m < $members; $m++)
						{		
					
							$memberx++;
							
							// if the member could not be saved
							if ( !demodata_create_group_member( $newgroupid, 0, $membersx ) )
							{
								$success = false;
								$error = "Error creating group member " . $memberx;
								$memberx--;
								// break out of this loop
								break;
							}
						}
					}
				}
			}
		
			if ($success)
			{
			
				echo '
				<div class="demodatasuccess">
				<p>' . $groupx . " " . __("demo groups created") . '</p>
				<p>' . $memberx . ' ' . __("demo group members created") . '</p>
				<p><a href="http://' . $current_site->domain . '">' . __("Your system is now ready for testing here.") . '</a></p>
				</div>
				';
				
			} else {
			
				echo '
				<div class="demodataerror">
				<p>' . $groupx . ' ' . __("demo groups created") . '</p>
				<p>' . $memberx . ' ' . __("demo group members created") . '</p>
				<p>' . __("Errors encountered:") . ' ' . $error . '</p>
				<p><a href="http://' . $current_site->domain . '">' . __("You can test this data in your system here.") . '</a></p>
				</div>
				';
				
			}
		} else {
		
			echo '
			<div class="demodataerror">
			<p>' . __("Some of your settings were not valid. Please check all the settings below.") . '</p>
			</div>
			';
		
		}
	} else {
		
		echo '
		<div class="demodataerror">
		<p>' . __("BuddyPress not found.") . '</p>
		</div>
		';
	
	}
}

// create demo friends
function demodata_create_friends()
{

	// detect BuddyPress
	$buddypress = defined( 'BP_ROOT_BLOG' );
	
	if ($buddypress)
	{

		global $wpdb;
		global $current_site;
		
		// get friends settings
		$maxfriends = @$_POST["maxfriends"] == "" ? 50 : (int)$_POST["maxfriends"];
		$maxfriends = $maxfriends > 100 ? $maxfriends = 100 : $maxfriends = $maxfriends;
		
		// check all the settings
		if (
			$maxfriends != ""
		)
		{
			$go = true;
		} else {
			$go = false;
		}
		
		// if the settings are OK
		if ($go)
		{
		
			$friendsx = 0;
			$success = true;
			
			// get users
			$sql = "select id from " . $wpdb->users . " order by id;";
			$users = $wpdb->get_results($sql);

			// loop users
			foreach ($users as $user)
			{
			
				// get a random mumber of friends
				$friends = rand(1, $maxfriends);
				
				// loop the required number of friends
				for ($f = 0; $f < $friends; $f++)
				{
					$friendx++;
				
					// create the friendship
					if (!demodata_create_friendship($user->id))
					{
						$success = false;
						$error = "Error creating friendship " . $friendx;
						$friendx--;
						break;
					}
				
				}
				
			}
		
			if ($success)
			{
			
				echo '
				<div class="demodatasuccess">
				<p>' . $friendx . " " . __("demo friendships created") . '</p>
				<p><a href="http://' . $current_site->domain . '">' . __("Your system is now ready for testing here.") . '</a></p>
				</div>
				';
				
			} else {
			
				echo '
				<div class="demodataerror">
				<p>' . $friendx . ' ' . __("demo friendships created") . '</p>
				<p>' . __("Errors encountered:") . ' ' . $error . '</p>
				<p><a href="http://' . $current_site->domain . '">' . __("You can test this data in your system here.") . '</a></p>
				</div>
				';
				
			}
		} else {
		
			echo '
			<div class="demodataerror">
			<p>' . __("Some of your settings were not valid. Please check all the settings below.") . '</p>
			</div>
			';
		
		}
	} else {
		
		echo '
		<div class="demodataerror">
		<p>' . __("BuddyPress not found.") . '</p>
		</div>
		';
	
	}
}

// create demo wire messages
function demodata_create_wire()
{
	// detect BuddyPress
	$buddypress = defined( 'BP_ROOT_BLOG' );
	
	if ($buddypress)
	{

		global $wpdb;
		global $current_site;
		
		// get wire settings
		$maxwire = @$_POST["maxwire"] == "" ? 25 : (int)$_POST["maxwire"];
		$maxwire = $maxwire > 50 ? $maxwire = 50 : $maxwire = $maxwire;
		
		// check all the settings
		if (
			$maxwire != ""
		)
		{
			$go = true;
		} else {
			$go = false;
		}
		
		// if the settings are OK
		if ($go)
		{
		
			$friendsx = 0;
			$success = true;
			
			// get users
			$sql = "select id from " . $wpdb->users . " order by id;";
			$users = $wpdb->get_results($sql);
			
			// loop users
			foreach ($users as $user)
			{
			
				// get a random number of wire messages
				$wires = rand(0, $maxwire);
				
				// loop the number of required wires
				for($w = 0; $w < $wires; $w++)
				{
					$wirex++;
					// see if the wire can be created
					if (!demodata_create_wire_message($user->id))
					{
						// continue
						$wirex--;
					}
				}
				
			}
		
			if ($success)
			{
			
				echo '
				<div class="demodatasuccess">
				<p>' . $wirex . " " . __("demo wire messages created") . '</p>
				<p><a href="http://' . $current_site->domain . '">' . __("Your system is now ready for testing here.") . '</a></p>
				</div>
				';
				
			} else {
			
				echo '
				<div class="demodataerror">
				<p>' . $wirex . ' ' . __("demo wire messages created") . '</p>
				<p>' . __("Errors encountered:") . ' ' . $error . '</p>
				<p><a href="http://' . $current_site->domain . '">' . __("You can test this data in your system here.") . '</a></p>
				</div>
				';
				
			}
		} else {
		
			echo '
			<div class="demodataerror">
			<p>' . __("Some of your settings were not valid. Please check all the settings below.") . '</p>
			</div>
			';
		
		}
	} else {
		
		echo '
		<div class="demodataerror">
		<p>' . __("BuddyPress not found.") . '</p>
		</div>
		';
	
	}
}

// ======================================================
// Data creation functions
// ======================================================

// create a bookmark
function demodata_create_link($blogdomain, $linkx)
{
	$link = array(
		'link_id' => 0,
		'link_name' => 'Bookmark ' . $linkx,
		'link_url' => 'http://' . $blogdomain . '/#bookmark' . $linkx,
		'link_rating' => 0
	);
	return wp_insert_link($link);
}

// create a comments
function demodata_create_comment($blogdomain, $postid, $commentx)
{
	$commentcontent = demodata_generate_random_text(1000);
	$time = current_time('mysql', $gmt = 0); 
	$comment = array(
	    'comment_post_ID' => $postid,
	    'comment_author' => 'Commenter '.$commentx,
	    'comment_author_email' => 'commenter@' . $blogdomain,
	    'comment_author_url' => 'http://commenter.url',
	    'comment_content' => $commentcontent,
	    'comment_type' => '',
	    'comment_parent' => 0,
	    'user_ID' => 0,
	    'comment_author_IP' => '127.0.0.1',
	    'comment_agent' => 'Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.0.10) Gecko/2009042316 Firefox/3.0.10 (.NET CLR 3.5.30729)',
	    'comment_date' => $time,
	    'comment_date_gmt' => $time,
	    'comment_approved' => 1,
	);
	return wp_insert_comment($comment);
}

// create a category
function demodata_create_category($blogdomain, $categoryx)
{
	$cat = ucfirst(strtolower(demodata_generate_random_text(20)));
	return wp_create_category( $cat );
}

// create a post
function demodata_create_post($blogdomain, $maxpostlength, $postx)
{
	// generate the random post data
	$postcontent = demodata_generate_html(rand(1, $maxpostlength));
	// generate array of category ids
	$cats = demodata_random_categories($blogdomain);
	$post = array('post_status' => 'live',
	'post_type' => 'post',
	'post_author' => $userid,
	'ping_status' => 'open',
	'post_parent' => 0,
	'menu_order' => 0,
	'to_ping' => '',
	'pinged' => '',
	'post_password' => '',
	'guid' => 'http://' . $blogdomain . '/post' . $postx,
	'post_content_filtered' => '',
	'post_excerpt' => '',
	'import_id' => 0,
	'post_status' => 'publish',
	'post_title' => 'Demo post ' . $postx,
	'post_content' => $postcontent,
	'post_excerpt' => '',
	'post_category' => $cats);
	return wp_insert_post($post);
}

// create a page
function demodata_create_page($blogdomain, $parentid, $maxpagelength, $pagex)
{
	$pagecontent = demodata_generate_html(rand(1, $maxpagelength));
	$page = array('post_status' => 'live',
	'post_type' => 'page',
	'post_author' => 1,
	'ping_status' => 'open',
	'post_parent' => $parentid,
	'menu_order' => 0,
	'to_ping' => '',
	'pinged' => '',
	'post_password' => '',
	'guid' => 'http://' . $blogdomain . '/page' . $pagex,
	'post_content_filtered' => '',
	'post_excerpt' => '',
	'import_id' => 0,
	'post_status' => 'publish',
	'post_title' => 'Demo page ' . $pagex,
	'post_content' => $pagecontent,
	'post_excerpt' => '',
	'post_category' => $cats);
	return wp_insert_post( $page );
}

// create a blog (taken from /wp-admin/wpmu-edit.php)
function demodata_create_blog($newid, $blogdomain, $blogname, $user_id)
{
	global $current_site;
	global $current_user;
	global $wpdb;
	
	$base = "/";
	
	if( constant('VHOST') == 'yes' ) {
		$newdomain = $blogdomain.".".$current_site->domain;
		$path = $base;
	} else {
		$newdomain = $current_site->domain;
		$path = $base.$blogdomain.'/';
	}
	
	// install this blog
	$meta = apply_filters('signup_create_blog_meta', array ('lang_id' => 1, 'public' => 1));
	$id = wpmu_create_blog($newdomain, $path, $blogname, $user_id , $meta, $current_site->id);

	if( !is_wp_error($id) ) {
		if( get_user_option( $user_id, 'primary_blog' ) == 1 )
			update_user_option( $user_id, 'primary_blog', $id, true );
			
		// add the user to the blog
		add_user_to_blog($id, $user_id, 'administrator');
			
		return $id;
	} else {
		return false;
	}
}

// ======================================================
// BuddyPress functions
// ======================================================

// create a wire message
function demodata_create_wire_message($userid)
{
	global $wpdb;

	$message = demodata_generate_random_text(100);
	
	return bp_wire_new_post( 1, $message, "wire", false,  $wpdb->base_prefix . "user_" . $userid . "_wire");
}

// create a BuddyPress group
function demodata_create_group($groupname, $num)
{
	// set up the new group
	$group_obj = new BP_Groups_Group();
	$group_obj->name = $groupname;
	$group_obj->slug = sanitize_title( $groupname );
	$group_obj->description = "Demo group " . $num . " created by Demo Data Creator plugin.";
	$group_obj->status = 'public';
	$group_obj->is_invitation_only = 0;
	$group_obj->enable_wire = 1;
	$group_obj->enable_forum = 1;
	$group_obj->enable_photos = 1;
	$group_obj->photos_admin_only = 0;
	$group_obj->date_created = time();
	
	// save the group
	$saved = $group_obj->save();
	
	if ( $saved )
	{
		return $group_obj->id;
	} else {
		return false;
	}
}

// create a BuddyPress group member
function demodata_create_group_member($group_id, $admin, $num)
{	
	$user_id = demodata_random_user_id();
	return groups_join_group( $group_id, $user_id );
}

// create a BuddyPress friendship
function demodata_create_friendship($userid)
{
	return friends_add_friend($userid, demodata_random_user_id($userid), true);
}

// ======================================================
// Helper functions
// ======================================================

// get a random user id
function demodata_random_user_id($id = 0)
{
	global $wpdb;
	return $wpdb->get_var("select id from " . $wpdb->users . " where " . $wpdb->escape($id) . " = 0 or id <> " . $wpdb->escape($id) . " order by rand() limit 1;");
}

// delete demo data
function demodata_delete()
{
	global $wpdb;
	global $current_site;

	echo '
	<div class="demodatasuccess">
	<h2>' . __("Deleting demo data...") . '</h2>
	<ul>
	';
	
	$i = 0;
	
	// delete user meta
	$sql = "delete from " . $wpdb->usermeta . " where user_id > 1;";
	$users = $wpdb->query($sql);
	
	// count users
	$sql = "select count(id) from " . $wpdb->users . " where id > 1;";
	$usercount = $wpdb->get_var($sql);
	
	// delete users
	$sql = "delete from " . $wpdb->users . " where id > 1;";
	$users = $wpdb->query($sql);
	
	// alter auto integer
	$sql = "ALTER TABLE " . $wpdb->users . " AUTO_INCREMENT = 2;";
	$users = $wpdb->query($sql);

	echo '<li>' . $usercount . ' ' . ("users deleted") . '</li>
	';
	
	$i = 0;
	
	// count blogs
	$sql = "select count(blog_id) from " . $wpdb->blogs . " where blog_id > 1;";
	$blogcount = $wpdb->get_var($sql);
	
	// delete blogs
	$sql = "select blog_id from " . $wpdb->blogs . " where blog_id > 1;";
	$blogs = $wpdb->get_results($sql);
	foreach($blogs as $blog)
	{
		if (wpmu_delete_blog($blog->blog_id, true))
		{
			$i++;
		}
	}
	
	// alter auto integer
	$sql = "ALTER TABLE " . $wpdb->blogs . " AUTO_INCREMENT = 2;";
	$users = $wpdb->query($sql);

	echo '<li>' . $blogcount . ' ' . __("blogs deleted") . '</li>
	';
	
	$i = 0;
	
	// detect BuddyPress
	$buddypress = defined( 'BP_ROOT_BLOG' );
	
	if ($buddypress)
	{
		// delete user activity
		$sql = "delete from " . $wpdb->base_prefix. "bp_activity_user_activity;";
		$user_activity = $wpdb->query($sql);
		
		// delete user activity cached
		$sql = "delete from " . $wpdb->base_prefix. "bp_activity_user_activity_cached;";
		$user_activity_cached = $wpdb->query($sql);
		
		// delete site activity
		$sql = "delete from " . $wpdb->base_prefix. "bp_activity_sitewide;";
		$sitewide_activity = $wpdb->query($sql);
	
		// delete user meta
		$sql = "delete from " . $wpdb->base_prefix. "bp_xprofile_data;";
		$xprofile_data = $wpdb->query($sql);
	
		// delete group meta
		$sql = "delete from " . $wpdb->base_prefix. "bp_groups_groupmeta;";
		$groups_groupmeta = $wpdb->query($sql);
		
		// delete group membership
		$sql = "delete from " . $wpdb->base_prefix. "bp_groups_members;";
		$groups_members = $wpdb->query($sql);
	
		// delete group wire
		$sql = "delete from " . $wpdb->base_prefix. "bp_groups_wire;";
		$groups_wire = $wpdb->query($sql);
	
		// delete groups
		$sql = "delete from " . $wpdb->base_prefix. "bp_groups;";
		$groups = $wpdb->query($sql);
		
		// delete friendships
		$sql = "delete from " . $wpdb->base_prefix. "bp_friends;";
		$friends = $wpdb->query($sql);
		
		// delete user blog posts
		$sql = "delete from " . $wpdb->base_prefix. "bp_user_blogs_posts;";
		$user_blogs_posts = $wpdb->query($sql);
		
		// delete user blog comments
		$sql = "delete from " . $wpdb->base_prefix. "bp_user_blogs_comments;";
		$user_blogs_comments = $wpdb->query($sql);
		
		echo '<li>' . __("BuddyPress data deleted") . '</li>
		';
	}
	
	echo '
	</ul>
	</div>
	';
}

// watch for a form action
function demodata_watch_form()
{
	// if submitting form
	if (is_array($_POST) && count($_POST) > 0 && isset($_POST["action"]))
	{	
		set_time_limit(300);
	
		if ($_POST["action"] == "create")
		{
			demodata_create();
		}
		
		if ($_POST["action"] == "delete")
		{
			demodata_delete();
		}
	}
}

// ======================================================
// Admin forms
// ======================================================

// write out the form
function demodata_form()
{
	global $current_site;
	global $wpdb;

	// get the upgrade functions
	require_once( ABSPATH . 'wp-admin/includes/upgrade.php');

	// detect BuddyPress
	$buddypress = defined( 'BP_ROOT_BLOG' );

	echo '
	<div class="wrap">
	';
	
	if (function_exists('populate_queries'))
	{
	
	demodata_watch_form();
	
	echo '
	
		<h2>' . __("Create demo data") . '</h2>
		<p>' . __("Use the form below to create multiple test users and blog in this WPMU system. Warning: this may take some time if you are creating a lot of data.") . '</p>
		
		<form action="wpmu-admin.php?page=demodata_form&amp;create=users" method="post" class="demodata">
		<fieldset>
		
			<h4>' . __("Users") . '</h4>
			
			<p><label for="users">' . __("Number of users (max 1000)") . '</label>
			<input type="text" name="users" id="users" value="100" /></p>
			
			<p><label for="useremailtemplate">' . __("User email template (with [x] for the number)") . '</label>
			<input type="text" name="useremailtemplate" id="useremailtemplate" value="demouser[x]@' . $current_site->domain . '" class="text" /></p>
			
			<p><label for="createusers">' . __("Create users") . '</label>
			<input type="hidden" name="create" value="users" />
			<input type="hidden" name="action" value="create" />
			<button class="button" id="createusers">' . __("Create users") . '</button></p>
			
		</fieldset>
		</form>
		
		<form action="wpmu-admin.php?page=demodata_form&amp;create=blogs" method="post" class="demodata">
		<fieldset>
		
			<h4>' . __("Blogs") . '</h4>
			
			<p><label for="maxblogsperuser">' . __("Maximum number of blogs per user (max 5)") . '</label>
			<input type="text" name="maxblogsperuser" id="maxblogsperuser" value="1" /></p>
			
			<p><label for="membershiptype1">' . __("All users must have at least one blog") . '</label>
			<input type="radio" name="membershiptype" id="membershiptype1" value="1" checked="checked" /></p>
			
			<p><label for="membershiptype2">' . __("Users may have zero or more blogs") . '</label>
			<input type="radio" name="membershiptype" id="membershiptype2" value="2" /></p>
			
			<p><label for="createblogs">' . __("Create blogs") . '</label>
			<input type="hidden" name="create" value="blogs" />
			<input type="hidden" name="action" value="create" />
			<button class="button" id="createblogs">' . __("Create blogs") . '</button></p>
			
		</fieldset>
		</form>
		
		<form action="wpmu-admin.php?page=demodata_form&amp;create=categories" method="post" class="demodata">
		<fieldset>
		
			<h4>' . __("Categories") . '</h4>
			
			<p><label for="maxblogcategories">' . __("Maximum number of categories per blog (max 25)") . '</label>
			<input type="text" name="maxblogcategories" id="maxblogcategories" value="10" /></p>
			
			<p><label for="createcategories">' . __("Create categories") . '</label>
			<input type="hidden" name="create" value="categories" />
			<input type="hidden" name="action" value="create" />
			<button class="button" id="createcategories">' . __("Create categories") . '</button></p>
			
		</fieldset>
		</form>
		
		<form action="wpmu-admin.php?page=demodata_form&amp;create=posts" method="post" class="demodata">
		<fieldset>
		
			<h4>' . __("Posts") . '</h4>
			
			<p><label for="maxblogposts">' . __("Maximum number of posts per blog (max 100)") . '</label>
			<input type="text" name="maxblogposts" id="maxblogposts" value="50" /></p>
			
			<p><label for="maxpostlength">' . __("Maximum number of blog post paragraphs (min 1, max 50)") . '</label>
			<input type="text" name="maxpostlength" id="maxpostlength" value="10" /></p>
			
			<p><label for="createposts">' . __("Create posts") . '</label>
			<input type="hidden" name="create" value="posts" />
			<input type="hidden" name="action" value="create" />
			<button class="button" id="createposts">' . __("Create posts") . '</button></p>
			
		</fieldset>
		</form>
		
		<form action="wpmu-admin.php?page=demodata_form&amp;create=pages" method="post" class="demodata">
		<fieldset>
		
			<h4>' . __("Pages") . '</h4>
			
			<p><label for="maxpages">' . __("Maximum number of pages per blog (max 50)") . '</label>
			<input type="text" name="maxpages" id="maxpages" value="25" /></p>
			
			<p><label for="maxtoppages">' . __("Maximum number of top-level pages per blog (max 10)") . '</label>
			<input type="text" name="maxtoppages" id="maxtoppages" value="5" /></p>
			
			<p><label for="maxpageslevels">' . __("Maximum number of level to nest pages (max 5)") . '</label>
			<input type="text" name="maxpageslevels" id="maxpageslevels" value="3" /></p>
			
			<p><label for="maxpagelength">' . __("Maximum number of blog page paragraphs (min 1, max 50)") . '</label>
			<input type="text" name="maxpagelength" id="maxpagelength" value="10" /></p>
			
			<p><label for="createpages">' . __("Create pages") . '</label>
			<input type="hidden" name="create" value="pages" />
			<input type="hidden" name="action" value="create" />
			<button class="button" id="createpages">' . __("Create pages") . '</button></p>
			
		</fieldset>
		</form>
			
		<form action="wpmu-admin.php?page=demodata_form&amp;create=comments" method="post" class="demodata">
		<fieldset>
		
				<h4>' . __("Comments") . '</h4>
			
			<p><label for="maxcomments">' . __("Maximum number of comments per post (max 50)") . '</label>
			<input type="text" name="maxcomments" id="maxcomments" value="10" /></p>
			
			<p><label for="createcomments">' . __("Create comments") . '</label>
			<input type="hidden" name="create" value="comments" />
			<input type="hidden" name="action" value="create" />
			<button class="button" id="createcomments">' . __("Create comments") . '</button></p>
			
		</fieldset>
		</form>
			
		<form action="wpmu-admin.php?page=demodata_form&amp;create=blogs" method="post" class="demodata">
		<fieldset>
		
			<h4>' . __("Links") . '</h4>
			
			<p><label for="maxbloglinks">' . __("Maximum number of links in blogroll (max 100)") . '</label>
			<input type="text" name="maxbloglinks" id="maxbloglinks" value="25" /></p>
			
			<p><label for="createlinks">' . __("Create links") . '</label>
			<input type="hidden" name="create" value="links" />
			<input type="hidden" name="action" value="create" />
			<button class="button" id="createlinks">' . __("Create links") . '</button></p>
			
		</fieldset>
		</form>
		';
		
		// if Buddypress has been detected
		if ($buddypress)
		{
		echo '	
		
		<form action="wpmu-admin.php?page=demodata_form&amp;create=groups" method="post" class="demodata">
		<fieldset>
		
			<h4>' . __("BuddyPress Groups") . '</h4>
			
			<p><label for="maxgroups">' . __("Number of groups (maximum 500)") . '</label>
			<input type="text" name="maxgroups" id="maxgroups" value="100" /></p>
			
			<p><label for="maxgroupmembership">' . __("Max. number of groups per user (maximum 50)") . '</label>
			<input type="text" name="maxgroupmembership" id="maxgroupmembership" value="25" /></p>
			
			<p><label for="creategroups">' . __("Create groups") . '</label>
			<input type="hidden" name="create" value="groups" />
			<input type="hidden" name="action" value="create" />
			<button class="button" id="creategroups">' . __("Create groups") . '</button></p>
			
		</fieldset>
		</form>
		
		<!--
		creating demo wire messages disabled as I can\'t get it to work
		<form action="wpmu-admin.php?page=demodata_form&amp;create=wire" method="post" class="demodata">
		<fieldset>
		
			<h4>' . __("BuddyPress Wire Messages") . '</h4>
			
			<p><label for="maxwire">' . __("Number of wire messages per user (maximum 50)") . '</label>
			<input type="text" name="maxwire" id="maxwire" value="25" /></p>
			
			<p><label for="createwire">' . __("Create wire messages") . '</label>
			<input type="hidden" name="create" value="wire" />
			<input type="hidden" name="action" value="create" />
			<button class="button" id="createwire">' . __("Create wire") . '</button></p>
			
		</fieldset>
		</form>
		-->
		
		<form action="wpmu-admin.php?page=demodata_form&amp;create=friends" method="post" class="demodata">
		<fieldset>
			
			<h4>' . __("BuddyPress Friends") . '</h4>
			
			<p><label for="maxfriends">' . __("Number of friends per user (maximum 100)") . '</label>
			<input type="text" name="maxfriends" id="maxfriends" value="50" /></p>
		
			<p>' . __("Creating this demo data may take several minutes. Please be patient, and only click the button below once.") . '</p>
			
			<p><label for="createfriends">' . __("Create friends") . '</label>
			<input type="hidden" name="create" value="friends" />
			<input type="hidden" name="action" value="create" />
			<button class="button" id="createfriends">' . __("Create friends") . '</button></p>
		
		</fieldset>
		</form>
		
		';
		}
		
		echo '		
		<h3>Delete demo data</h3>
		
		<form action="wpmu-admin.php?page=demodata_form" method="post" class="demodata">
		<fieldset>
		
		';
		if ($buddypress)
		{
			echo '
			<p>Delete all user and blog data in your database except for information and tables for blog ID 1 and user ID 1. This will also delete all Buddypress groups and friend relationships.</p>
			';
		} else {
			echo '
			<p>Delete all user and blog data in your database except for information and tables for blog ID 1 and user ID 1.</p>
			';
		}
		echo '
			<p><label for="delete">Delete demo data</label>
			<input type="hidden" name="action" value="delete" />
			<button class="button" name="delete" id="delete">Delete</button></p>
		
		</fieldset>
		</form>
	';
	
	} else {
	
		echo '
		<h2>' . __("populate_queries() function not found") . '</h2>
		<p>' . __("Sorry, the function populate_queries() was not found. This function is required to use the Demo Data Creator plugin. See this Wordpress MU ticket for technical details: ") . '<a href="http://trac.mu.wordpress.org/ticket/394">http://trac.mu.wordpress.org/ticket/394</a>.</p>
		';
	
	}
	
	echo '
	</div>
	';
}

// ======================================================
// Content functions
// ======================================================
 
// return a random array of category ids
function demodata_random_categories()
{
	$limit = rand(1, 6);
	global $wpdb;
	return $wpdb->get_col("select term_id from ".$wpdb->terms." order by rand() limit ".$limit.";");
}

// return a random contact first name
function demodata_firstname()
{
	$firstnames = explode(",", "Alan,Albert,Allen,Amy,Andrew,Angela,Anita,Ann,Anne,Annette,Anthony,Arthur,Barbara,Barry,Beth,Betty,Beverly,Bill,Billy,Bobby,Bonnie,Bradley,Brenda,Brian,Bruce,Bryan,Carl,Carol,Carolyn,Catherine,Cathy,Charles,Cheryl,Chris,Christine,Christopher,Cindy,Connie,Craig,Curtis,Cynthia,Dale,Daniel,Danny,Darlene,Darryl,David,Dawn,Dean,Debbie,Deborah,Debra,Denise,Dennis,Diana,Diane,Donald,Donna,Dorothy,Douglas,Edward,Elizabeth,Ellen,Eric,Frank,Gail,Gary,George,Gerald,Glenn,Gloria,Greg,Gregory,Harold,Henry,Jack,Jacqueline,James,Jane,Janet,Janice,Jay,Jean,Jeff,Jeffery,Jeffrey,Jennifer,Jerry,Jill,Jim,Jimmy,Joan,Joanne,Joe,John,Johnny,Jon,Joseph,Joyce,Judith,Judy,Julie,Karen,Katharine,Kathleen,Kathryn,Kathy,Keith,Kelly,Kenneth,Kevin,Kim,Kimberly,Larry,Laura,Laurie,Lawrence,Leslie,Linda,Lisa,Lori,Lynn,Margaret,Maria,Mark,Martha,Martin,Mary,Matthew,Michael,Michele,Michelle,Mike,Nancy,Pamela,Patricia,Patrick,Paul,Paula,Peggy,Peter,Philip,Phillip,Ralph,Randall,Randy,Raymond,Rebecca,Renee,Rhonda,Richard,Rick,Ricky,Rita,Robert,Robin,Rodney,Roger,Ronald,Ronnie,Rose,Roy,Russell,Ruth,Samuel,Sandra,Scott,Sharon,Sheila,Sherry,Shirley,Stephanie,Stephen,Steve,Steven,Susan,Suzanne,Tammy,Teresa,Terri,Terry,Theresa,Thomas,Tim,Timothy,Tina,Todd,Tom,Tony,Tracy,Valerie,Vicki,Vickie,Vincent,Walter,Wanda,Wayne,Wendy,William,Willie");
	return $firstnames[ array_rand( $firstnames, 1 ) ];
}

// return a random contact last name
function demodata_lastname()
{
	$lastnames = explode(",", "Smith,Johnson,Williams,Jones,Brown,Davis,Miller,Wilson,Moore,Taylor,Anderson,Thomas,Jackson,White,Harris,Martin,Thompson,Garcia,Robinson,Clark,Lewis,Lee,Walker,Hall,Allen,Young,King,Wright,Lopez,Hill,Scott,Green,Adams,Baker,Nelson,Carter,Mitchell,Perez,Roberts,Turner,Phillips,Campbell,Parker,Evans,Edwards,Collins,Stewart,Morris,Rogers,Reed,Cook,Morgan,Bell,Murphy,Bailey,Rivera,Cooper,Richardson,Cox,Howard,Ward,Peterson,Gray,James,Watson,Brooks,Kelly,Sanders,Price,Bennett,Wood,Barnes,Ross,Henderson,Coleman,Jenkins,Perry,Powell,Long,Patterson,Hughes,Flores,Washington,Butler,Simmons,Foster,Bryant,Alexander,Russell,Griffin,Hayes,Myers,Ford,Hamilton,Graham,Sullivan,Wallace,Woods,Cole,West,Jordan,Owens,Reynolds,Fisher,Ellis,Harrison,Gibson,Mcdonald,Cruz,Marshall,Gomez,Murray,Freeman,Wells,Webb,Simpson,Stevens,Tucker,Porter,Hunter,Hicks,Crawford,Henry,Boyd,Mason,Morales,Kennedy,Warren,Dixon,Ramos,Reyes,Burns,Gordon,Shaw,Holmes,Rice,Robertson,Hunt,Black,Daniels,Palmer,Mills,Nichols,Grant,Knight,Ferguson,Rose,Stone,Hawkins,Dunn,Perkins,Hudson,Spencer,Gardner,Stephens,Payne,Pierce,Berry,Matthews,Arnold,Wagner,Willis,Ray,Watkins,Olson,Carroll,Duncan,Snyder,Hart,Cunningham,Bradley,Lane,Andrews,Ruiz,Harper,Fox,Riley,Armstrong,Carpenter,Weaver,Greene,Lawrence,Elliott,Chavez,Sims,Austin,Peters,Kelley,Franklin,Lawson,Fields,Ryan,Schmidt,Carr,Castillo,Wheeler,Chapman,Oliver,Montgomery,Richards,Williamson,Johnston,Banks,Meyer,Bishop,Mccoy,Howell,Morrison,Hansen,Garza,Harvey,Little,Burton,Stanley,Nguyen,George,Jacobs,Reid,Kim,Fuller,Lynch,Dean,Gilbert,Garrett,Welch,Larson,Frazier,Burke,Hanson,Day,Moreno,Bowman,Fowler");
	return $lastnames[ array_rand( $lastnames, 1 ) ];
}

// return a random blog name
function demodata_blogname()
{
	$names1 = explode(",", "View from,All about,Here is,About,Writings on,See my,Blog about,I love,Do you like,Unlimited,,,");
	$names2 = explode(",", "Photos,Food,History,Music,Sport,Football,Rugby,Golf,Tennis,Cricket,Hockey,Jazz,Folk,Rock,Metal,Victoriana,Elizabethan,Stamp,Model,Flying Astronomy,Robotics,Bird watching,Calligraphy,Drawing,Dollhouses,Knitting,Antiques,Book,Animation,Gardening,Literature");
	return $names1[ array_rand( $names1, 1 ) ] . ' ' . $names2[ array_rand( $names2, 1 ) ];
}

// return a random group name
function demodata_groupname()
{
	$names1 = explode(",", "Photos,Food,History,Music,Sport,Football,Rugby,Golf,Tennis,Cricket,Hockey,Jazz,Folk,Rock,Metal,Victoriana,Elizabethan,Stamp,Model,Flying Astronomy,Robotics,Bird watching,Calligraphy,Drawing,Dollhouses,Knitting,Antiques,Book,Animation,Gardening,Literature");
	$names2 = explode(",", "Professionals,Lovers,Heroes,Fans,Appreciators,Association,Society,Community,World,Group,Order,Gang,Guild,,,");
	return $names1[ array_rand( $names1, 1 ) ] . ' ' . $names2[ array_rand( $names2, 1 ) ];
}

// generate random html content
function demodata_generate_html($maxblocks = 4)
{
	$head = "<h1>HTML Ipsum Presents (" . $maxblocks . " blocks)</h1>";
	$htmlstr = '	       
<p><strong>Pellentesque habitant morbi tristique</strong> senectus et netus et malesuada fames ac turpis egestas. Vestibulum tortor quam, feugiat vitae, ultricies eget, tempor sit amet, ante. Donec eu libero sit amet quam egestas semper. <em>Aenean ultricies mi vitae est.</em> Mauris placerat eleifend leo. Quisque sit amet est et sapien ullamcorper pharetra. Vestibulum erat wisi, condimentum sed, <code>commodo vitae</code>, ornare sit amet, wisi. Aenean fermentum, elit eget tincidunt condimentum, eros ipsum rutrum orci, sagittis tempus lacus enim ac dui. <a href="#">Donec non enim</a> in turpis pulvinar facilisis. Ut felis.</p>
<!--break-->
<h2>Header Level 2</h2>
	       
<ol>
   <li>Lorem ipsum dolor sit amet, consectetuer adipiscing elit.</li>
   <li>Aliquam tincidunt mauris eu risus.</li>
</ol>
<!--break-->
<blockquote><p>Lorem ipsum dolor sit amet, consectetur adipiscing elit. Vivamus magna. Cras in mi at felis aliquet congue. Ut a est eget ligula molestie gravida. Curabitur massa. Donec eleifend, libero at sagittis mollis, tellus est malesuada tellus, at luctus turpis elit sit amet quam. Vivamus pretium ornare est.</p></blockquote>
<!--break-->
<h3>Header Level 3</h3>

<ul>
   <li>Lorem ipsum dolor sit amet, consectetuer adipiscing elit.</li>
   <li>Aliquam tincidunt mauris eu risus.</li>
</ul>
<!--break-->
<pre><code>
#header h1 a { 
	display: block; 
	width: 300px; 
	height: 80px; 
}
</code></pre>
<!--break-->
<table summary="Table summary">
   <caption>Table Caption</caption>
   <thead>
      <tr>
         <th>Header</th>
         <th>Header</th>
         <th>Header</th>
      </tr>
   </thead>
   <tbody>
      <tr>
         <td>Content</td>
         <td>1</td>
         <td>a</td>
      </tr>
      <tr>
         <td>Content</td>
         <td>2</td>
         <td>b</td>
      </tr>
   </tbody>
</table><table summary="Table summary">
   <caption>Table Caption</caption>
   <thead>
      <tr>
         <th>Header</th>
         <th>Header</th>
         <th>Header</th>
      </tr>
   </thead>
   <tbody>
      <tr>
         <td>Content</td>
         <td>1</td>
         <td>a</td>
      </tr>
      <tr>
         <td>Content</td>
         <td>2</td>
         <td>b</td>
      </tr>
   </tbody>
</table>
<!--break-->
<p>Pellentesque habitant morbi tristique senectus et netus et malesuada fames ac turpis egestas. Vestibulum tortor quam, feugiat vitae, ultricies eget, tempor sit amet, ante. Donec eu libero sit amet quam egestas semper. Aenean ultricies mi vitae est. Mauris placerat eleifend leo. Quisque sit amet est et sapien ullamcorper pharetra. Vestibulum erat wisi, condimentum sed, commodo vitae, ornare sit amet, wisi. Aenean fermentum, elit eget tincidunt condimentum, eros ipsum rutrum orci, sagittis tempus lacus enim ac dui. Donec non enim in turpis pulvinar facilisis. Ut felis. Praesent dapibus, neque id cursus faucibus, tortor neque egestas augue, eu vulputate magna eros eu erat. Aliquam erat volutpat. Nam dui mi, tincidunt quis, accumsan porttitor, facilisis luctus, metus</p>
<!--break-->
<dl>
   <dt>Definition list</dt>
   <dd>Consectetur adipisicing elit, sed do eiusmod tempor incididunt ut labore et dolore magna 
aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea 
commodo consequat.</dd>
   <dt>Lorem ipsum dolor sit amet</dt>
   <dd>Consectetur adipisicing elit, sed do eiusmod tempor incididunt ut labore et dolore magna 
aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea 
commodo consequat.</dd>
</dl>
<!--break-->
<p>Pellentesque habitant morbi tristique senectus et netus et malesuada fames ac turpis egestas. Vestibulum tortor quam, feugiat vitae, ultricies eget, tempor sit amet, ante. Donec eu libero sit amet quam egestas semper. Aenean ultricies mi vitae est. Mauris placerat eleifend leo.</p>';

	$htmlstr = explode("<!--break-->", $htmlstr);
	$blocks = count($htmlstr);
	
	if ($maxblocks > count($htmlstr)) { $maxblocks = $blocks+1; }

	$out = "";
	
	for($x = 0; $x < $maxblocks; $x++)
	{
		$out .= $htmlstr[rand(0, $blocks)] . "\n\n";
	}
	
	return $out;
}

// generate random text content
function demodata_generate_random_text($maxlength, $randomstart = true)
{
	$len = rand(0, $maxlength);

	$str = 'Lorem ipsum dolor sit amet, consectetur adipiscing elit. Donec tincidunt dignissim risus. Sed mattis quam non ante. Nullam condimentum fringilla nibh. Quisque non lectus. Phasellus est orci, fermentum sit amet, vulputate sit amet, facilisis varius, nulla. In aliquet. Sed justo lectus, tempor eleifend, vulputate nec, auctor ac, nibh. Cras sem turpis, adipiscing quis, ultricies et, blandit vitae, justo. Suspendisse potenti. Quisque dapibus neque. Nulla purus sapien, interdum ac, lobortis quis, tempus quis, tellus. Cras semper odio id purus. Nunc eros. In massa. Curabitur viverra, felis eget sagittis consectetur, neque neque fringilla lectus, quis faucibus risus tellus consequat nunc. In nec lectus. Aliquam erat volutpat. Ut vel odio. Quisque lacus. Etiam consectetur rutrum justo.

Duis facilisis. Aliquam sagittis. Proin consectetur egestas metus. Curabitur pellentesque posuere arcu. Integer lorem nulla, congue a, rhoncus nec, vestibulum a, ante. Mauris lobortis iaculis erat. Maecenas faucibus tincidunt dui. Cras accumsan vestibulum ligula. Morbi dapibus, lorem nec euismod pharetra, augue risus congue augue, molestie eleifend lacus magna nec magna. Morbi ac lorem.

Aliquam id ipsum. Sed mauris. Cum sociis natoque penatibus et magnis dis parturient montes, nascetur ridiculus mus. Proin ac ipsum. Nunc vitae sapien sit amet tellus eleifend hendrerit. Phasellus orci nunc, fermentum eu, ultrices quis, dapibus vitae, lectus. Donec blandit ligula vitae justo. Quisque rutrum ante ac nisi. Sed at mi. Donec eget odio in est tempus malesuada. Donec sem. Donec tortor. In auctor purus a lectus. Nulla a leo. Curabitur velit. In et lorem. Quisque congue, lorem sed cursus ullamcorper, mauris urna adipiscing enim, eget pulvinar est lacus ut augue.

Aliquam erat volutpat. Proin scelerisque lectus non purus. Donec quis tellus. Vestibulum dictum imperdiet lacus. Integer magna libero, feugiat id, tincidunt eu, venenatis vel, mi. Cras mi. Vestibulum est. Pellentesque habitant morbi tristique senectus et netus et malesuada fames ac turpis egestas. Sed ut magna quis felis congue commodo. Fusce dictum hendrerit dolor. Donec commodo euismod dolor. Curabitur nibh. Vestibulum ut risus. Integer laoreet, ipsum congue congue suscipit, ipsum est molestie dolor, consectetur tempus justo lectus ac mauris.

Integer vulputate molestie quam. Vestibulum et nisi. Nullam in magna quis libero posuere vestibulum. Donec mi leo, elementum ut, tincidunt at, condimentum eu, est. Donec sed nisl ac justo consectetur viverra. Nulla volutpat est vitae nisl. Nullam aliquam ipsum a arcu facilisis adipiscing. Pellentesque cursus. Cum sociis natoque penatibus et magnis dis parturient montes, nascetur ridiculus mus. Nulla gravida viverra urna.

Aenean condimentum, arcu sit amet volutpat ullamcorper, erat sapien tempor dolor, ac consectetur diam ante at nibh. Proin erat lectus, vehicula id, consequat quis, semper ac, felis. Sed in lectus. Duis nec massa. Nullam augue. Duis dolor felis, porta et, molestie vitae, imperdiet eget, purus. Mauris iaculis. In cursus, neque eu sollicitudin ullamcorper, odio mauris tempus odio, id tincidunt metus leo vel ipsum. Quisque suscipit. Class aptent taciti sociosqu ad litora torquent per conubia nostra, per inceptos himenaeos. Nullam sed libero. Nunc sollicitudin diam ac dui. Nunc faucibus auctor tortor. Quisque ipsum sem, hendrerit accumsan, congue ut, porttitor ut, turpis. Sed sollicitudin, leo et condimentum tempus, massa augue dictum est, iaculis posuere nulla felis quis tortor. Pellentesque ac nisl vitae nunc porta pretium. Praesent id erat. Cras leo. Quisque eleifend metus nec lorem.

Sed sed quam non mauris aliquam rhoncus. Vestibulum ante ipsum primis in faucibus orci luctus et ultrices posuere cubilia Curae; Praesent quam turpis, dignissim et, consequat in, porttitor eget, orci. Aenean pretium, orci vel ultrices dapibus, ipsum metus lobortis ante, ac tincidunt urna erat et tortor. Mauris nisl eros, dapibus et, tristique eget, dignissim non, lacus. Fusce aliquam, turpis quis varius blandit, purus elit sollicitudin leo, vitae posuere odio odio id dui. Fusce adipiscing. Maecenas a enim eu sem accumsan laoreet. Donec sem eros, egestas ornare, fermentum quis, malesuada sed, risus. Sed eleifend faucibus magna. Fusce malesuada ante eget massa. Donec consectetur dolor vitae erat. Lorem ipsum dolor sit amet, consectetur adipiscing elit. Maecenas elit risus, pellentesque sed, imperdiet quis, pulvinar in, odio.

Duis nulla diam, fringilla at, feugiat et, tincidunt consectetur, magna. Duis id est ac neque mollis ullamcorper. Nulla quis urna. Ut vitae nisi sit amet lectus blandit ultrices. Vestibulum ante ipsum primis in faucibus orci luctus et ultrices posuere cubilia Curae; Phasellus consectetur nunc at sapien. Ut pellentesque laoreet diam. Morbi arcu neque, congue rhoncus, sollicitudin ut, condimentum id, lectus. Nulla facilisi. Praesent neque lacus, pretium eu, molestie at, laoreet convallis, metus. Fusce leo nisi, ornare ut, ullamcorper ac, molestie et, quam. In vehicula arcu eu risus. Fusce ultrices lectus sit amet diam. Maecenas commodo risus eu diam. Maecenas at nibh. Nullam mattis pharetra dolor. Donec eleifend leo sit amet diam. Quisque ante. Aliquam erat volutpat.

Maecenas vel lectus. Maecenas convallis lorem at risus. Nullam facilisis tortor. Quisque quis turpis. Suspendisse consectetur nisl. Integer facilisis, massa consectetur mattis sollicitudin, lectus elit pharetra eros, ut dignissim sapien nunc eu nisi. Nunc tortor. Vestibulum imperdiet. Aliquam erat volutpat. Integer fermentum tincidunt nisl. Vivamus fringilla, augue vel consectetur convallis, justo leo rutrum dui, et dapibus ipsum nulla sit amet est. Curabitur augue. Sed nec magna vel est auctor porttitor. Integer at lorem. Sed et turpis nec lorem consectetur volutpat. Donec vestibulum cursus mauris. Nullam blandit urna quis nibh. Proin sollicitudin elementum tellus. Suspendisse mauris enim, ultricies ut, rhoncus non, sodales at, arcu.

Nam eu tortor. Nam venenatis congue nibh. Donec posuere lacinia neque. Pellentesque vehicula. Nam eleifend ipsum. Vestibulum lectus diam, viverra vitae, tempor sit amet, eleifend eu, quam. Aenean aliquam ornare tellus. Donec vel ligula ut mauris pellentesque mattis. In neque leo, porta non, rutrum vel, sollicitudin a, libero. Pellentesque ornare mauris id odio. Aliquam erat volutpat. Aenean eget ante eget nunc feugiat lacinia. Duis ullamcorper consequat risus. Nunc at magna.

Aliquam in elit vitae dui gravida venenatis. Cum sociis natoque penatibus et magnis dis parturient montes, nascetur ridiculus mus. Aliquam bibendum risus id nunc. Nullam tempus molestie eros. Maecenas molestie pharetra augue. In sagittis enim vitae libero. Praesent feugiat blandit lacus. Lorem ipsum dolor sit amet, consectetur adipiscing elit. Maecenas erat lacus, sollicitudin placerat, facilisis nec, tempor nec, elit. Pellentesque iaculis urna ut leo mattis sagittis. Cras urna. Vestibulum malesuada orci eget leo.

Etiam egestas auctor sapien. Cum sociis natoque penatibus et magnis dis parturient montes, nascetur ridiculus mus. Praesent feugiat. Donec vehicula, dui nec adipiscing ultricies, lorem urna eleifend arcu, eu congue libero lectus ac lacus. Lorem ipsum dolor sit amet, consectetur adipiscing elit. Praesent pulvinar pellentesque lacus. Suspendisse urna orci, rutrum a, placerat a, suscipit a, est. Aliquam ac odio. Fusce arcu lorem, sodales et, condimentum ultrices, fringilla ut, tellus. Quisque velit justo, molestie ac, imperdiet et, vulputate semper, dolor. Phasellus laoreet, sapien in molestie gravida, velit quam consequat enim, sit amet malesuada arcu justo vitae turpis. Aliquam orci lorem, ornare ut, congue nec, bibendum at, erat. Donec vitae nisl id risus euismod viverra. Pellentesque eleifend risus sit amet diam. Mauris pharetra ornare tellus. Pellentesque ante elit, vestibulum eget, laoreet dictum, pulvinar vitae, velit. Mauris vel augue quis enim ornare imperdiet.

Phasellus sed tellus eget lacus molestie laoreet. Phasellus commodo euismod mauris. Sed facilisis nulla a est. Proin sit amet lectus. Morbi suscipit libero a nisl. Sed quam ipsum, ullamcorper non, congue vitae, ornare non, nisl. Nunc ultrices. Aliquam imperdiet velit sit amet nulla. Sed a neque. Fusce tempus tortor ut diam. Quisque sagittis lacus eget velit. Quisque augue magna, commodo in, molestie nec, convallis at, tortor. Etiam blandit ultrices tortor. Aliquam nisi risus, lobortis vitae, elementum pulvinar, viverra vel, dolor. Donec metus urna, faucibus aliquet, egestas in, adipiscing sit amet, risus. Vestibulum ante ipsum primis in faucibus orci luctus et ultrices posuere cubilia Curae; Proin nisi. Nunc hendrerit nisi lobortis purus.

Vivamus turpis ante, ultrices scelerisque, elementum id, tempor at, quam. Cras eu mauris eu nulla congue convallis. Fusce ornare, nibh sit amet porta rhoncus, nibh metus tincidunt magna, quis gravida dui neque pharetra risus. Etiam a felis. Fusce sed dolor. Mauris lectus mi, fringilla tempor, varius sit amet, placerat a, velit. Suspendisse justo. In vehicula urna fringilla neque. Suspendisse cursus, magna a imperdiet pellentesque, lacus velit dignissim urna, vel suscipit massa enim ac nunc. Proin porta aliquet eros. Curabitur ut erat. Quisque vitae tortor.

Duis imperdiet, mi eget euismod fermentum, odio nisl posuere quam, sit amet tristique urna diam at lacus. Duis congue lacus non ipsum. Donec felis tortor, lacinia at, rhoncus id, scelerisque ornare, nisi. Mauris felis ligula, pharetra vitae, posuere eget, tincidunt at, turpis. Donec eget ligula. Praesent fermentum dictum nisl. Phasellus enim. Nam placerat. Ut dignissim est nec lorem. Aliquam eros augue, rutrum et, placerat in, euismod in, libero. Morbi venenatis, eros non gravida rhoncus, leo metus venenatis augue, vitae dignissim lorem sapien eget diam. Ut nisl. Fusce quis lorem. Etiam mollis risus. Integer luctus. In quis quam. Pellentesque habitant morbi tristique senectus et netus et malesuada fames ac turpis egestas metus.';

	if (!$randomstart)
	{

		$out = trim(substr($str, 0, $len));
	
	} else {
	
		$out = trim(substr($str, rand(0 + $len, strlen($str) - $len), $len));
	
	}

	return $out;
}
?>